#!/usr/bin/env python
#
# yum-plugin-repolist 0.0.3
#
# Copyright (c) 2007 Daniel de Kok
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You can install this plugin by copying it to /usr/lib/yum-plugins. To
# enable this plugin, make sure that you have 'plugins=1' in /etc/yum.conf,
# and create the file /etc/yum/pluginconf.d/repolist.conf with the
# following content:
#
# [main]
# enabled=1
#
# Please report errors to Daniel de Kok <daniel@centos.org>

import yum.plugins
import yum.logginglevels
import yum.Errors

requires_api_version = '2.5'
plugin_type = (yum.plugins.TYPE_INTERACTIVE,)

class RepoProtectionInfo:
	"""This class provides repository protection information."""
	def __init__(self, pluginList):
		if 'priorities' in pluginList:
			self.plugin = 'priorities'
		elif 'protectbase' in pluginList:
			self.plugin = 'protectbase'
		else:
			self.plugin = None

	def header(self):
		if self.plugin == 'priorities':
			return "priority"
		elif self.plugin == 'protectbase':
			return "protectbase"
		return ""

	def value(self, repo):
		if self.plugin == 'priorities':
			return repo.priority
		elif self.plugin == 'protectbase':
			return repo.protect 
		return ""

	def sortFun(self, val1, val2):
		if self.plugin == 'priorities':
			return cmp(val1.priority, val2.priority)
		elif self.plugin == 'protectbase':
			return cmp(val2.protect, val1.protect)
		else:
			return 0

class RepolistCommand:
	"""This class implements a 'repolist' command for yum."""
	def getNames(self):
		return ['repolist']

	def getUsage(self):
		return 'Show a list of repositories and their status'

	def doCheck(self, base, basecmd, extcmds):
		pass

	def doCommand(self, base, basecmd, extcmds):
		try:
			allRepos = base.repos.repos.values()
		except yum.Errors.YumBaseError, e:
			return 1, [str(e)]

		protectInfo = RepoProtectionInfo(base.plugins._plugins)

		# Formatting from cli.py
		format_string = "%-15.15s %-35.35s %-15.15s %s"

		# Output list headers
		base.verbose_logger.log(yum.logginglevels.INFO_2, "\n" + format_string,
			'repo id', 'repo name', 'status', protectInfo.header())
		base.verbose_logger.log(yum.logginglevels.INFO_2, format_string,
			'=======', '=========', '======', '=' * len(protectInfo.header()))

		allRepos.sort(protectInfo.sortFun)
		for repo in allRepos:
			if repo.enabled:
				base.verbose_logger.log(yum.logginglevels.INFO_2, format_string,
					repo, repo.name, 'enabled', protectInfo.value(repo))
			else:
				base.verbose_logger.log(yum.logginglevels.INFO_2, format_string,
					repo, repo.name, 'disabled', protectInfo.value(repo))
		return 0, ['']

def config_hook(conduit):
	conduit.registerCommand(RepolistCommand()) 

